@@ -33,6 +33,7 @@
 #include "camel-exchange-search.h"
 #include "camel-exchange-store.h"
 #include "camel-exchange-summary.h"
+#include "camel-exchange-journal.h"
 
 #include <camel/camel-data-wrapper.h>
 #include <camel/camel-exception.h>
@@ -44,7 +45,7 @@
 #include <camel/camel-stream-filter.h>
 #include <camel/camel-stream-mem.h>
 
-static CamelFolderClass *parent_class = NULL;
+static CamelOfflineFolderClass *parent_class = NULL;
 
 /* Returns the class for a CamelFolder */
 #define CF_CLASS(so) CAMEL_FOLDER_CLASS (CAMEL_OBJECT_GET_CLASS(so))
@@ -52,15 +53,10 @@ static CamelFolderClass *parent_class = NULL;
 static void refresh_info (CamelFolder *folder, CamelException *ex);
 static void folder_sync (CamelFolder *folder, gboolean expunge,
 			 CamelException *ex);
-static void expunge             (CamelFolder *folder,
-				 CamelException *ex);
+static void exchange_expunge (CamelFolder *folder, CamelException *ex);
 static void append_message (CamelFolder *folder, CamelMimeMessage *message,
 			    const CamelMessageInfo *info, char **appended_uid,
 			    CamelException *ex);
-static gboolean set_message_flags (CamelFolder *folder, const char *uid,
-				   guint32 flags, guint32 set);
-static void set_message_user_tag (CamelFolder *folder, const char *uid,
-				  const char *name, const char *value);
 static CamelMimeMessage *get_message         (CamelFolder *folder,
 					      const gchar *uid,
 					      CamelException *ex);
@@ -85,24 +81,26 @@ static void   transfer_messages_the_hard_way (CamelFolder *source,
 					      GPtrArray **transferred_uids,
 					      gboolean delete_originals,
 					      CamelException *ex);
+static void exchange_refresh_info (CamelFolder *folder, CamelException *ex);
+static void exchange_sync (CamelFolder *folder, gboolean expunge, CamelException *ex);
 
 static void
 class_init (CamelFolderClass *camel_folder_class)
 {
-	parent_class = (CamelFolderClass *)camel_type_get_global_classfuncs (camel_folder_get_type ());
+	parent_class = CAMEL_OFFLINE_FOLDER_CLASS (camel_type_get_global_classfuncs (camel_offline_folder_get_type ())) ;
 
 	/* virtual method definition */
 	camel_folder_class->sync = folder_sync;
 	camel_folder_class->refresh_info = refresh_info;
-	camel_folder_class->expunge = expunge;
+	camel_folder_class->expunge = exchange_expunge;
 	camel_folder_class->append_message = append_message;
-	camel_folder_class->set_message_flags = set_message_flags;
-	camel_folder_class->set_message_user_tag = set_message_user_tag;
 	camel_folder_class->get_message = get_message;
 	camel_folder_class->search_by_expression = search_by_expression;
 	camel_folder_class->search_by_uids = search_by_uids;
 	camel_folder_class->search_free = search_free;
 	camel_folder_class->transfer_messages_to = transfer_messages_to;
+	camel_folder_class->refresh_info = exchange_refresh_info;
+	camel_folder_class->sync = exchange_sync;
 }
 
 #define CAMEL_EXCHANGE_SERVER_FLAGS \
@@ -145,7 +143,8 @@ camel_exchange_folder_get_type (void)
 
 	if (camel_exchange_folder_type == CAMEL_INVALID_TYPE) {
 		camel_exchange_folder_type = camel_type_register (
-			CAMEL_FOLDER_TYPE, "CamelExchangeFolder",
+			camel_offline_folder_get_type (),
+			"CamelExchangeFolder",
 			sizeof (CamelExchangeFolder),
 			sizeof (CamelExchangeFolderClass),
 			(CamelObjectClassInitFunc) class_init,
@@ -179,7 +178,7 @@ folder_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
 	uids = g_ptr_array_new ();
 	for (i = 0; i < summary->len; i++) {
 		info = summary->pdata[i];
-		if (!(info->flags & CAMEL_MESSAGE_DELETED))
+		if (!(camel_message_info_flags(info) & CAMEL_MESSAGE_DELETED))
 			continue;
 		g_ptr_array_add (uids, (char *)camel_message_info_uid (info));
 	}
@@ -207,7 +206,7 @@ refresh_info (CamelFolder *folder, CamelException *ex)
 }
 
 static void
-expunge (CamelFolder *folder, CamelException *ex)
+exchange_expunge (CamelFolder *folder, CamelException *ex)
 {
 	CamelExchangeFolder *exch = CAMEL_EXCHANGE_FOLDER (folder);
 	CamelFolder *trash;
@@ -246,7 +245,7 @@ append_message_data (CamelFolder *folder, GByteArray *message,
 
 	if (camel_stub_send (exch->stub, ex, CAMEL_STUB_CMD_APPEND_MESSAGE,
 			     CAMEL_STUB_ARG_FOLDER, folder->full_name,
-			     CAMEL_STUB_ARG_UINT32, info->flags,
+			     CAMEL_STUB_ARG_UINT32, camel_message_info_flags(info),
 			     CAMEL_STUB_ARG_STRING, subject,
 			     CAMEL_STUB_ARG_BYTEARRAY, message,
 			     CAMEL_STUB_ARG_RETURN,
@@ -274,7 +273,12 @@ append_message (CamelFolder *folder, CamelMimeMessage *message,
 		CamelException *ex)
 {
 	CamelStream *stream_mem;
+	CamelOfflineStore *offline = (CamelOfflineStore *) folder->parent_store;
 
+	if (offline->state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL) {
+		camel_exchange_journal_append ((CamelExchangeJournal *) ((CamelExchangeFolder *)folder)->journal, message, info, appended_uid, ex);
+		return;
+	}
 	stream_mem = camel_stream_mem_new ();
 	camel_data_wrapper_write_to_stream (CAMEL_DATA_WRAPPER (message),
 					    stream_mem);
@@ -287,48 +291,6 @@ append_message (CamelFolder *folder, CamelMimeMessage *message,
 	camel_object_unref (CAMEL_OBJECT (stream_mem));
 }
 
-static gboolean
-set_message_flags (CamelFolder *folder, const char *uid,
-		   guint32 flags, guint32 set)
-{
-	CamelExchangeFolder *exch = CAMEL_EXCHANGE_FOLDER (folder);
-	gboolean changed;
-
-	if (folder->permanent_flags == 0)
-		return FALSE;
-
-	changed = CAMEL_FOLDER_CLASS (parent_class)->set_message_flags (folder, uid, flags, set);
-	if (!changed)
-		return FALSE;
-
-	camel_stub_send_oneway (exch->stub, CAMEL_STUB_CMD_SET_MESSAGE_FLAGS,
-				CAMEL_STUB_ARG_FOLDER, folder->full_name,
-				CAMEL_STUB_ARG_STRING, uid,
-				CAMEL_STUB_ARG_UINT32, set,
-				CAMEL_STUB_ARG_UINT32, flags,
-				CAMEL_STUB_ARG_END);
-	return TRUE;
-}
-
-static void
-set_message_user_tag (CamelFolder *folder, const char *uid,
-		      const char *name, const char *value)
-{
-	CamelExchangeFolder *exch = CAMEL_EXCHANGE_FOLDER (folder);
-
-	if (folder->permanent_flags == 0)
-		return;
-
-	CAMEL_FOLDER_CLASS (parent_class)->set_message_user_tag (folder, uid, name, value);
-
-	camel_stub_send_oneway (exch->stub, CAMEL_STUB_CMD_SET_MESSAGE_TAG,
-				CAMEL_STUB_ARG_FOLDER, folder->full_name,
-				CAMEL_STUB_ARG_STRING, uid,
-				CAMEL_STUB_ARG_STRING, name,
-				CAMEL_STUB_ARG_STRING, value,
-				CAMEL_STUB_ARG_END);
-}
-
 static void
 fix_broken_multipart_related (CamelMimePart *part)
 {
@@ -559,12 +521,12 @@ transfer_messages_the_hard_way (CamelFolder *source, GPtrArray *uids,
 
 		ba = get_message_data (source, uids->pdata[i], &local_ex);
 		if (!ba) {
-			camel_folder_summary_info_free (source->summary, info);
+			camel_message_info_free(info);
 			break;
 		}
 
 		append_message_data (dest, ba, NULL, info, &ret_uid, &local_ex);
-		camel_folder_summary_info_free (source->summary, info);
+		camel_message_info_free(info);
 		g_byte_array_free (ba, TRUE);
 
 		if (camel_exception_is_set (&local_ex))
@@ -628,12 +590,42 @@ transfer_messages_to (CamelFolder *source, GPtrArray *uids,
 {
 	CamelExchangeFolder *exch_source = CAMEL_EXCHANGE_FOLDER (source);
 	CamelExchangeFolder *exch_dest = CAMEL_EXCHANGE_FOLDER (dest);
+	CamelOfflineStore *offline = (CamelOfflineStore *) source->parent_store;
+	CamelMessageInfo *info;
 	GPtrArray *ret_uids = NULL;
-	int hier_len;
+	int hier_len, i;
 
 	camel_operation_start (NULL, delete_originals ? _("Moving messages") :
 			       _("Copying messages"));
 
+	/* Check for offline operation */
+	if (offline->state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL) {
+		CamelExchangeJournal *journal = (CamelExchangeJournal *) exch_dest->journal;
+		CamelMimeMessage *message;
+		
+		for (i = 0; i < uids->len; i++) {
+			info = camel_folder_summary_uid (source->summary, uids->pdata[i]);
+			if (!info)
+				continue;
+			
+			if (!(message = get_message (source, camel_message_info_uid (info), ex)))
+				break;
+
+			camel_exchange_journal_transfer (journal, (CamelExchangeFolder *)source, 
+							message, info, uids->pdata[i], NULL, ex);
+			camel_object_unref (message);
+
+			if (camel_exception_is_set (ex))
+				break;
+			
+			if (delete_originals)
+				camel_folder_set_message_flags (source, camel_message_info_uid (info),
+					CAMEL_MESSAGE_DELETED, CAMEL_MESSAGE_DELETED);
+		}
+		goto end;
+ 	}
+
+
 	hier_len = strcspn (source->full_name, "/");
 	if (strncmp (source->full_name, dest->full_name, hier_len) != 0) {
 		transfer_messages_the_hard_way (source, uids, dest,
@@ -665,7 +657,7 @@ transfer_messages_to (CamelFolder *source, GPtrArray *uids,
 		}
 	} else if (transferred_uids)
 		*transferred_uids = NULL;
-
+end:
 	camel_operation_end (NULL);
 }
 
@@ -725,7 +717,7 @@ camel_exchange_folder_add_message (CamelExchangeFolder *exch,
 
 	info = camel_folder_summary_uid (folder->summary, uid);
 	if (info) {
-		camel_folder_summary_info_free (folder->summary, info);
+		camel_message_info_free(info);
 		return;
 	}
 
@@ -738,27 +730,25 @@ camel_exchange_folder_add_message (CamelExchangeFolder *exch,
 	einfo = (CamelExchangeMessageInfo *)info;
 
 	if (einfo->thread_index) {
-		g_hash_table_insert (exch->thread_index_to_message_id,
-				     g_strdup (einfo->thread_index),
-				     g_memdup (&info->message_id, sizeof (CamelSummaryMessageID)));
+		CamelSummaryMessageID *parent;
 
-		if (!info->references) {
-			CamelSummaryMessageID *parent;
+		if (einfo->info.message_id.id.id)
+			g_hash_table_insert (exch->thread_index_to_message_id,
+					     g_strdup (einfo->thread_index),
+					     g_memdup (&einfo->info.message_id, sizeof (CamelSummaryMessageID)));
 
-			parent = find_parent (exch, einfo->thread_index);
-			if (parent) {
-				info->references = g_new (CamelSummaryReferences, 1);
-				info->references->size = 1;
-				memcpy (&info->references->references[0], parent,
-					sizeof (info->references->references[0]));
-			}
+		parent = find_parent (exch, einfo->thread_index);
+		if (parent && einfo->info.references == NULL) {
+			einfo->info.references = g_malloc(sizeof(CamelSummaryReferences));
+			memcpy(&einfo->info.references->references[0], parent, sizeof(*parent));
+			einfo->info.references->size = 1;
 		}
 	}
 	camel_object_unref (CAMEL_OBJECT (msg));
 
-	camel_message_info_set_uid (info, g_strdup (uid));
-	info->flags = flags;
-	info->size = size;
+	info->uid = g_strdup (uid);
+	einfo->info.flags = flags;
+	einfo->info.size = size;
 
 	camel_folder_summary_add (folder->summary, info);
 
@@ -805,7 +795,7 @@ camel_exchange_folder_remove_message (CamelExchangeFolder *exch,
 	}
 
 	camel_folder_summary_remove (summary, info);
-	camel_folder_summary_info_free (summary, info);
+	camel_message_info_free (info);
 	camel_data_cache_remove (exch->cache, "cache", uid, NULL);
 
 	changes = camel_folder_change_info_new ();
@@ -844,10 +834,10 @@ camel_exchange_folder_update_message_flags (CamelExchangeFolder *exch,
 					    const char *uid, guint32 flags)
 {
 	CamelFolder *folder = CAMEL_FOLDER (exch);
-	CamelMessageInfo *info;
+	CamelMessageInfoBase *info;
 	CamelFolderChangeInfo *changes;
 
-	info = camel_folder_summary_uid (folder->summary, uid);
+	info = (CamelMessageInfoBase *)camel_folder_summary_uid (folder->summary, uid);
 	if (!info)
 		return;
 
@@ -882,10 +872,10 @@ camel_exchange_folder_update_message_tag (CamelExchangeFolder *exch,
 					  const char *value)
 {
 	CamelFolder *folder = CAMEL_FOLDER (exch);
-	CamelMessageInfo *info;
+	CamelMessageInfoBase *info;
 	CamelFolderChangeInfo *changes;
 
-	info = camel_folder_summary_uid (folder->summary, uid);
+	info = (CamelMessageInfoBase *)camel_folder_summary_uid (folder->summary, uid);
 	if (!info)
 		return;
 
@@ -922,7 +912,7 @@ camel_exchange_folder_construct (CamelFolder *folder, CamelStore *parent,
 {
 	CamelExchangeFolder *exch = (CamelExchangeFolder *)folder;
 	const char *short_name;
-	char *summary_file;
+	char *summary_file, *journal_file;
 	GPtrArray *summary, *uids;
 	GByteArray *flags;
 	guint32 folder_flags;
@@ -943,7 +933,7 @@ camel_exchange_folder_construct (CamelFolder *folder, CamelStore *parent,
 	}
 
 	summary_file = g_strdup_printf ("%s/summary", folder_dir);
-	folder->summary = camel_exchange_summary_new (summary_file);
+	folder->summary = camel_exchange_summary_new (folder, summary_file);
 	g_free (summary_file);
 	if (!folder->summary) {
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
@@ -960,6 +950,16 @@ camel_exchange_folder_construct (CamelFolder *folder, CamelStore *parent,
 		return FALSE;
 	}
 
+	journal_file = g_strdup_printf ("%s/journal", folder_dir);
+	exch->journal = camel_exchange_journal_new (exch, journal_file);
+	g_free (journal_file);
+	if (!exch->journal) {
+		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
+					_("Could not create journal for %s"),
+					name);
+		return FALSE;
+	}
+
 	exch->thread_index_to_message_id =
 		g_hash_table_new (g_str_hash, g_str_equal);
 
@@ -968,13 +968,13 @@ camel_exchange_folder_construct (CamelFolder *folder, CamelStore *parent,
 		info = camel_folder_summary_index (folder->summary, i);
 		einfo = (CamelExchangeMessageInfo *)info;
 
-		if (einfo->thread_index) {
+		if (einfo->thread_index && einfo->info.message_id.id.id) {
 			g_hash_table_insert (exch->thread_index_to_message_id,
 					     g_strdup (einfo->thread_index),
-					     g_memdup (&info->message_id, sizeof (CamelSummaryMessageID)));
+					     g_memdup (&einfo->info.message_id, sizeof (CamelSummaryMessageID)));
 		}
 
-		camel_folder_summary_info_free (folder->summary, info);
+		camel_message_info_free(info);
 	}
 
 	if (stub) {
@@ -991,8 +991,8 @@ camel_exchange_folder_construct (CamelFolder *folder, CamelStore *parent,
 		for (i = 0; i < summary->len; i++) {
 			info = summary->pdata[i];
 			uids->pdata[i] = (char *)camel_message_info_uid (info);
-			flags->data[i] = info->flags & CAMEL_EXCHANGE_SERVER_FLAGS;
-			camel_tag_list_free (&info->user_tags);
+			flags->data[i] = ((CamelMessageInfoBase *)info)->flags & CAMEL_EXCHANGE_SERVER_FLAGS;
+			camel_tag_list_free (&((CamelMessageInfoBase *)info)->user_tags);
 		}
 
 		camel_operation_start (NULL, _("Scanning for changed messages"));
@@ -1034,3 +1034,18 @@ camel_exchange_folder_construct (CamelFolder *folder, CamelStore *parent,
 
 	return TRUE;
 }
+
+static void 
+exchange_refresh_info (CamelFolder *folder, CamelException *ex)
+{
+	refresh_info (folder, ex);
+}
+
+static void 
+exchange_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
+{
+	if (expunge)
+		exchange_expunge (folder, ex);
+	
+	camel_folder_summary_save (folder->summary);
+}
