@@ -56,10 +56,12 @@ CamelProviderConfEntry exchange_conf_entries[] = {
 	{ CAMEL_PROVIDER_CONF_CHECKSPIN, "ad_limit", NULL,
 	  N_("Limit number of GAL responses: %s"), "y:1:500:10000" },
 	{ CAMEL_PROVIDER_CONF_SECTION_END },
-	{ CAMEL_PROVIDER_CONF_SECTION_START, "passwdexpiry", NULL,
-	  N_("Password Expiry Warning") },
+	{ CAMEL_PROVIDER_CONF_SECTION_START, "generals", NULL,
+	  N_("Options") },
 	{ CAMEL_PROVIDER_CONF_CHECKSPIN, "passwd_exp_warn_period", NULL,
 	  N_("Password Expiry Warning period: %s"), "y:1:7:90" },
+	{ CAMEL_PROVIDER_CONF_CHECKBOX, "offline_sync", NULL,
+	  N_("Automatically synchronize account locally"), "0" },
 	{ CAMEL_PROVIDER_CONF_SECTION_END },
 	{ CAMEL_PROVIDER_CONF_CHECKBOX, "filter", NULL,
 	  /* i18n: copy from evolution:camel-imap-provider.c */
@@ -71,6 +73,16 @@ CamelProviderConfEntry exchange_conf_entries[] = {
 	{ CAMEL_PROVIDER_CONF_END }
 };
 
+typedef gboolean (CamelProviderValidateUserFunc) (CamelURL *camel_url, const char *url, gboolean *remember_password, CamelException *ex);
+
+typedef struct {
+        CamelProviderValidateUserFunc *validate_user;
+}CamelProviderValidate;
+
+static gboolean exchange_validate_user_cb (CamelURL *camel_url, const char *owa_url, gboolean *remember_password, CamelException *ex);
+
+CamelProviderValidate validate_struct = { exchange_validate_user_cb };
+
 static CamelProvider exchange_provider = {
 	"exchange",
 	N_("Microsoft Exchange"),
@@ -82,9 +94,9 @@ static CamelProvider exchange_provider = {
 	CAMEL_PROVIDER_IS_REMOTE | CAMEL_PROVIDER_IS_SOURCE |
 	CAMEL_PROVIDER_IS_STORAGE | CAMEL_PROVIDER_IS_EXTERNAL,
 
-	CAMEL_URL_NEED_USER, 
+	CAMEL_URL_NEED_USER | CAMEL_URL_ALLOW_AUTH | CAMEL_URL_HIDDEN_HOST,
 
-	exchange_conf_entries,
+	exchange_conf_entries 
 
 	/* ... */
 };
@@ -126,12 +138,18 @@ exchange_auto_detect_cb (CamelURL *url, GHashTable **auto_detected,
 }
 
 static gboolean
-exchange_validate_user_cb (CamelURL *camel_url, char *owa_url, 
-			   CamelException *ex)
+exchange_validate_user_cb (CamelURL *camel_url, const char *owa_url, 
+			   gboolean *remember_password, CamelException *ex)
 {
 	gboolean valid;
+	char *host = NULL;
+
+	valid = e2k_validate_user (owa_url, camel_url->user, 
+				   &host, remember_password);
+
+	camel_url_set_host(camel_url, valid?host:NULL);
+	g_free(host);
 
-	valid = e2k_validate_user (owa_url, camel_url->user, &camel_url->host);
 	return valid;
 }
 
@@ -144,11 +162,12 @@ camel_provider_module_init (void)
 	exchange_provider.url_hash = exchange_url_hash;
 	exchange_provider.url_equal = exchange_url_equal;
 	exchange_provider.auto_detect = exchange_auto_detect_cb;
-	exchange_provider.validate_user = exchange_validate_user_cb;
 
 	bindtextdomain (GETTEXT_PACKAGE, CONNECTOR_LOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	exchange_provider.translation_domain = GETTEXT_PACKAGE;
+	exchange_provider.translation_domain = GETTEXT_PACKAGE;
+	exchange_provider.priv = (void *)&validate_struct;
 
 	camel_provider_register (&exchange_provider);
 }
