@@ -430,6 +430,12 @@ make_folder_info (CamelExchangeStore *exch, char *name, char *uri,
 	if (flags & CAMEL_STUB_FOLDER_NOSELECT)
 		info->flags = CAMEL_FOLDER_NOSELECT;
 
+	if (flags & CAMEL_STUB_FOLDER_SYSTEM)
+		info->flags |= CAMEL_FOLDER_SYSTEM;
+		
+	if (flags & CAMEL_STUB_FOLDER_TYPE_INBOX)
+		info->flags |= CAMEL_FOLDER_TYPE_INBOX;
+
 	return info;
 }
 
@@ -469,12 +475,12 @@ exchange_get_folder_info (CamelStore *store, const char *top, guint32 flags, Cam
 	GByteArray *folder_flags;
 	CamelFolderInfo *info;
 	int i;
-	
+#if 0	
 	if (((CamelOfflineStore *) store)->state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL) {
 		camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM, _("Cannot get folder info in offline mode."));
 		return NULL;
 	}
-	
+#endif	
 	RETURN_VAL_IF_NOT_CONNECTED (store, ex, NULL);
 
 	/* If the backend crashed, don't keep returning an error
