@@ -19,6 +19,13 @@
 
 /* e2k-autoconfig: Automatic account configuration backend code */
 
+/* Note on gtk-doc: Several functions in this file have intentionally-
+ * broken gtk-doc comments (that have only a single "*" after the
+ * opening "/") so that they can be overridden by versions in
+ * docs/reference/tmpl/e2k-autoconfig.sgml that use better markup.
+ * If you change the docs here, be sure to change them there as well.
+ */
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -36,6 +43,7 @@
 #include <resolv.h>
 
 #include "e2k-autoconfig.h"
+#include "e2k-validate.h"
 #include "e2k-encoding-utils.h"
 #include "e2k-context.h"
 #include "e2k-global-catalog.h"
@@ -47,10 +55,14 @@
 
 #include <e-util/e-account.h>
 #include <e-util/e-account-list.h>
+#include <e-util/e-passwords.h>
+#include <e-util/e-dialog-utils.h>
 #include <gconf/gconf-client.h>
 #include <libxml/tree.h>
 #include <libxml/HTMLparser.h>
 
+#include <gtk/gtk.h>
+
 static char *find_olson_timezone (const char *windows_timezone);
 static void set_account_uri_string (E2kAutoconfig *ac);
 
@@ -349,6 +361,8 @@ get_ctx_auth_handler (SoupMessage *msg, gpointer user_data)
  *   %E2K_AUTOCONFIG_FAILED: Other error.
  *
  * Return value: the new context, or %NULL
+ *
+ * (If you change this comment, see the note at the top of this file.)
  **/
 E2kContext *
 e2k_autoconfig_get_context (E2kAutoconfig *ac, E2kOperation *op,
@@ -598,6 +612,8 @@ static const int n_home_properties = sizeof (home_properties) / sizeof (home_pro
  *     e2k_autoconfig_check_exchange()
  *
  * Return value: an #E2kAutoconfigResult
+ *
+ * (If you change this comment, see the note at the top of this file.)
  **/
 E2kAutoconfigResult
 e2k_autoconfig_check_exchange (E2kAutoconfig *ac, E2kOperation *op)
@@ -803,7 +819,7 @@ find_global_catalog (E2kAutoconfig *ac)
 	return;
 }
 
-/*
+/**
  * e2k_autoconfig_get_global_catalog
  * @ac: an autoconfig context
  * @op: an #E2kOperation, for cancellation
@@ -851,6 +867,8 @@ e2k_autoconfig_get_global_catalog (E2kAutoconfig *ac, E2kOperation *op)
  *   %E2K_AUTOCONFIG_FAILED: Other error.
  *
  * Return value: an #E2kAutoconfigResult.
+ *
+ * (If you change this comment, see the note at the top of this file.)
  */
 E2kAutoconfigResult
 e2k_autoconfig_check_global_catalog (E2kAutoconfig *ac, E2kOperation *op)
@@ -1358,8 +1376,7 @@ read_config (void)
  * e2k_autoconfig_lookup_option:
  * @option: option name to look up
  *
- * Looks up an option in the config file. These values are NOT SIGNED.
- * The system administrator can put whatever values he wants here.
+ * Looks up an autoconfiguration hint in the config file (if present)
  *
  * Return value: the string value of the option, or %NULL if it is unset.
  **/
@@ -1370,3 +1387,169 @@ e2k_autoconfig_lookup_option (const char *option)
 		read_config ();
 	return g_hash_table_lookup (config_options, option);
 }
+
+static gboolean 
+validate (const char *owa_url, char *user, char *password, char **host)
+{
+	E2kAutoconfig *ac;
+	E2kOperation op;        /* FIXME */
+	E2kAutoconfigResult result;
+	gboolean valid = FALSE;
+	const char *old, *new;
+
+	ac = e2k_autoconfig_new (owa_url, user, password, 
+				 E2K_AUTOCONFIG_USE_EITHER);
+
+	e2k_operation_init (&op);
+	// e2k_autoconfig_set_gc_server (ac, ad_server, gal_limit) FIXME
+	// e2k_autoconfig_set_gc_server (ac, NULL, -1);
+	result = e2k_autoconfig_check_exchange (ac, &op);
+
+	if (result == E2K_AUTOCONFIG_OK) {
+		result = e2k_autoconfig_check_global_catalog (ac, &op);
+		e2k_operation_free (&op);
+		/* Need to fill GC server and return */
+		valid = TRUE;
+	}
+	else {
+		switch (result) {
+
+		case E2K_AUTOCONFIG_CANT_CONNECT:
+			if (!strncmp (ac->owa_uri, "http:", 5)) {
+				old = "http";
+				new = "https";
+			} else {
+				old = "https";
+				new = "http";
+			}
+
+			e_notice (NULL, GTK_MESSAGE_ERROR,
+				  _("Could not connect to the Exchange "
+			    	    "server.\nMake sure the URL is correct "
+			    	    "(try \"%s\" instead of \"%s\"?) "
+			    	    "and try again."), new, old);
+			valid = FALSE;
+			break;
+
+		case E2K_AUTOCONFIG_CANT_RESOLVE:
+			e_notice (NULL, GTK_MESSAGE_ERROR,
+			  	_("Could not locate Exchange server.\n"
+			    	  "Make sure the server name is spelled correctly "
+			    	  "and try again."));
+			valid = FALSE;
+			break;
+
+		case E2K_AUTOCONFIG_AUTH_ERROR:
+		case E2K_AUTOCONFIG_AUTH_ERROR_TRY_NTLM:
+		case E2K_AUTOCONFIG_AUTH_ERROR_TRY_BASIC:
+			e_notice (NULL, GTK_MESSAGE_ERROR,
+			  	_("Could not authenticate to the Exchange "
+			    	  "server.\nMake sure the username and "
+			    	  "password are correct and try again."));
+			valid = FALSE;
+			break;
+
+		case E2K_AUTOCONFIG_AUTH_ERROR_TRY_DOMAIN:
+			e_notice (NULL, GTK_MESSAGE_ERROR,
+			  	_("Could not authenticate to the Exchange "
+			    	  "server.\nMake sure the username and "
+			    	  "password are correct and try again.\n\n"
+			    	  "You may need to specify the Windows "
+			    	  "domain name as part of your username "
+			    	  "(eg, \"MY-DOMAIN\\%s\")."),
+			  	  ac->username);
+			valid = FALSE;
+			break;
+
+		case E2K_AUTOCONFIG_NO_OWA:
+		case E2K_AUTOCONFIG_NOT_EXCHANGE:
+			e_notice (NULL, GTK_MESSAGE_ERROR,
+			  	_("Could not find OWA data at the indicated URL.\n"
+			    	  "Make sure the URL is correct and try again."));
+			valid = FALSE;
+			break;
+
+		case E2K_AUTOCONFIG_CANT_BPROPFIND:
+			e_notice (
+				NULL, GTK_MESSAGE_ERROR,
+				_("Ximian Connector requires access to certain "
+			  	"functionality on the Exchange Server that appears "
+			  	"to be disabled or blocked.  (This is usually "
+			  	"unintentional.)  Your Exchange Administrator will "
+			  	"need to enable this functionality in order for "
+			  	"you to be able to use Ximian Connector.\n\n"
+			  	"For information to provide to your Exchange "
+			  	"administrator, please follow the link below:\n"
+				"http://www.ximian.com/products/connector/iis-lockdown.html, http://www.ximian.com/products/connector/iis-lockdown.html"));
+
+			valid = FALSE;
+			break;
+
+		case E2K_AUTOCONFIG_EXCHANGE_5_5:
+			e_notice (
+				NULL, GTK_MESSAGE_ERROR,
+				_("The Exchange server URL you provided is for an "
+			  	"Exchange 5.5 Server. Ximian Connector supports "
+			  	"Microsoft Exchange 2000 and 2003 only."));
+
+			valid = FALSE;
+			break;
+
+		default:
+			e_notice (NULL, GTK_MESSAGE_ERROR,
+			  	_("Could not configure Exchange account because "
+			    	  "an unknown error occurred. Check the URL, "
+			    	  "username, and password, and try again."));
+			valid = FALSE; /* FIXME return valid */
+			break;
+		}
+	}
+
+	*host = g_strdup (ac->pf_server);
+	return valid;
+}
+
+gboolean
+e2k_validate_user (const char *owa_url, char *user, 
+		   char **host, gboolean *remember_password)
+{
+	gboolean valid = FALSE, remember=FALSE;
+	char *key, *password, *prompt;
+
+	key = g_strdup_printf ("%s//%s@%s", "exchange:", user, owa_url); /* FIXME */
+	password = e_passwords_get_password ("Exchange", key);
+	if (!password) {
+		prompt = g_strdup_printf (_("Enter password for %s"), user);
+		password = e_passwords_ask_password (_("Enter password"),
+					"Exchange", key, prompt,
+					E_PASSWORDS_REMEMBER_FOREVER|E_PASSWORDS_SECRET,
+					&remember, NULL);
+		if (password) {
+			valid = validate (owa_url, user, password, host);
+			if (valid) {
+				//auto_detect_gc();
+
+				/* generate the proper key once the host name 
+				 * is read and remember password temporarily, 
+				 * so that at the end of * account creation, 
+				 * user will not be prompted, for password will
+				 * not be asked again. 
+				 */
+
+				*remember_password = remember;
+				g_free (key);
+				key = g_strdup_printf ("%s//%s@%s", 
+						       "exchange:", user, *host);
+				e_passwords_add_password (key, password);
+				e_passwords_remember_password ("Exchange", key);
+			}
+		}
+		g_free (prompt);
+	}
+	if (password && !valid) {
+		e_passwords_forget_password ("Exchange", key);
+	}
+	g_free (key);
+
+	return valid;
+}
